from django.conf import settings
from appconf import AppConf

class MyAppConf(AppConf):
    
    DEFAULT_FROM_EMAIL = None
    """
    The address messages appear to come from.  Can be overridden for a 
    particular template or when a message is prepared.
    """
    
    ALLOW_HTML_MESSAGES = False
    """
    If true, templates can produce HTML-formatted messages and provide 
    plain-text alternative content.
    """


from django.conf import settings
from django.contrib import admin
from django import forms

from models import EmailMessageTemplate
from forms import EmailListField


class EmailMessageTemplateAdmin(admin.ModelAdmin):
    readonly_fields = ('related_item_display',)
    
    def __init__(self, *args, **kwargs):
        super(EmailMessageTemplateAdmin, self).__init__(*args, **kwargs)
        self.exclude = ['content_type', 'object_id']
        if not settings.EMAILMESSAGETEMPLATES_ALLOW_HTML_MESSAGES:
            self.exclude.extend(['type','autogenerate_text','body_template_html']) 

    def formfield_for_dbfield(self, db_field, **kwargs):
        if db_field.name in ['base_cc', 'base_bcc',]:
            request = kwargs.pop("request", None)
            return db_field.formfield(form_class=EmailListField, **kwargs)
        return super(EmailMessageTemplateAdmin, self).formfield_for_dbfield(db_field, **kwargs)
    
    class Media:
        js = (
            'emailmessagetemplates/js/admin.js',
            )
    
admin.site.register(EmailMessageTemplate, EmailMessageTemplateAdmin)



import copy

from django.core.mail import get_connection
from django.conf import settings

from models import EmailMessageTemplate


def send_mail(name, related_object=None, context={}, from_email=None,
              recipient_list=[], fail_silently=False, auth_user=None,
               auth_password=None, connection=None):
    """
    Easy wrapper for sending a single templated message to a recipient list.  
    The template to use is retrieved from the database based on the name and 
    related_object (optional) fields.

    All members of the recipient list will see the other recipients in the 'To' 
    field.

    If auth_user is None, the EMAIL_HOST_USER setting is used.
    If auth_password is None, the EMAIL_HOST_PASSWORD setting is used.
    """

    template = EmailMessageTemplate.objects.get_template(name, related_object)

    connection = connection or get_connection(username=auth_user,
                                              password=auth_password,
                                              fail_silently=fail_silently)

    template.context=context
    template.from_email=from_email
    template.to=recipient_list
    template.connection=connection

    return template.send()


def send_mass_mail(name, related_object=None, datatuple=(), fail_silently=False,
                   auth_user=None, auth_password=None, connection=None):
    """
    Given a datatuple of (context, from_email, recipient_list), renders and 
    sends a message to each recipient list. Returns the number of emails sent.

    If from_email is None, the DEFAULT_FROM_EMAIL setting is used.
    If auth_user and auth_password are set, they're used to log in.
    If auth_user is None, the EMAIL_HOST_USER setting is used.
    If auth_password is None, the EMAIL_HOST_PASSWORD setting is used.

    If a related object is specified, we'll first try to retrieve a template 
    that matches both the name and the object, and if none exists, we'll try to 
    retrieve a template with the specified name without a related object instead
    (to support situations where some objects have a specialized template, but, 
    when none exists, we want to fall back to a default). 
    """

    template = EmailMessageTemplate.objects.get_template(name, related_object)

    connection = connection or get_connection(username=auth_user,
                                              password=auth_password,
                                              fail_silently=fail_silently)

    messages = []
    for (context, from_email, recipient_list) in datatuple:
        message = copy.deepcopy(template)
        message.context=context
        message.from_email=from_email
        message.to=recipient_list
        message.connection=connection
        messages.append(message)

    return connection.send_messages(messages)


def mail_admins(name, related_object=None, context={}, fail_silently=False,
                connection=None):
    """Sends a message to the admins, as defined by the ADMINS setting."""
    return send_mail(name, related_object, context, fail_silently=fail_silently,
                     recipient_list=[a[1] for a in settings.ADMINS],
                     connection=connection)


def mail_managers(name, related_object=None, context={}, fail_silently=False,
                connection=None):
    """Sends a message to the managers, as defined by the MANAGERS setting."""
    return send_mail(name, related_object, context, fail_silently=fail_silently,
                     recipient_list=[a[1] for a in settings.MANAGERS],
                     connection=connection)



from email.utils import getaddresses, formataddr

from django import forms
from django.utils.translation import ugettext_lazy as _


class EmailListField(forms.CharField):
    """
    A Django form field which validates a list of email addresses.
    Adapted from http://djangosnippets.org/snippets/1677/
    """
    default_error_messages = {
        'invalid': _('Please enter a valid list of e-mail addresses.')
    }

    def prepare_value(self, value):
        if isinstance(value, list) or isinstance(value, tuple):
            return ', '.join(value)
        return value
        
    def clean(self, value):
        value = super(EmailListField, self).clean(value)

        field = forms.EmailField()

        try:
            return [
                formataddr((name, field.clean(addr)))
            for name, addr in getaddresses([value])]
        except forms.ValidationError:
            raise forms.ValidationError(self.error_messages['invalid'])



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'EmailMessageTemplate'
        db.create_table(u'emailmessagetemplates_emailmessagetemplate', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('content_type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['contenttypes.ContentType'], null=True, blank=True)),
            ('object_id', self.gf('django.db.models.fields.PositiveIntegerField')(null=True, blank=True)),
            ('type', self.gf('django.db.models.fields.CharField')(default='text/plain', max_length=20)),
            ('subject_template', self.gf('django.db.models.fields.CharField')(max_length=2000)),
            ('body_template_html', self.gf('django.db.models.fields.TextField')(blank=True)),
            ('autogenerate_text', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('body_template', self.gf('django.db.models.fields.TextField')(blank=True)),
            ('sender', self.gf('django.db.models.fields.EmailField')(default='', max_length=75, blank=True)),
            ('base_cc', self.gf('emailmessagetemplates.fields.SeparatedValuesField')(default='', blank=True)),
            ('base_bcc', self.gf('emailmessagetemplates.fields.SeparatedValuesField')(default='', blank=True)),
            ('description', self.gf('django.db.models.fields.TextField')()),
            ('enabled', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('edited_date', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
            ('edited_user', self.gf('django.db.models.fields.TextField')(max_length=30, blank=True)),
        ))
        db.send_create_signal('emailmessagetemplates', ['EmailMessageTemplate'])

        # Adding unique constraint on 'EmailMessageTemplate', fields ['name', 'content_type', 'object_id']
        db.create_unique(u'emailmessagetemplates_emailmessagetemplate', ['name', 'content_type_id', 'object_id'])


    def backwards(self, orm):
        # Removing unique constraint on 'EmailMessageTemplate', fields ['name', 'content_type', 'object_id']
        db.delete_unique(u'emailmessagetemplates_emailmessagetemplate', ['name', 'content_type_id', 'object_id'])

        # Deleting model 'EmailMessageTemplate'
        db.delete_table(u'emailmessagetemplates_emailmessagetemplate')


    models = {
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'emailmessagetemplates.emailmessagetemplate': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('name', 'content_type', 'object_id'),)", 'object_name': 'EmailMessageTemplate'},
            'autogenerate_text': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'base_bcc': ('emailmessagetemplates.fields.SeparatedValuesField', [], {'default': "''", 'blank': 'True'}),
            'base_cc': ('emailmessagetemplates.fields.SeparatedValuesField', [], {'default': "''", 'blank': 'True'}),
            'body_template': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'body_template_html': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']", 'null': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {}),
            'edited_date': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'edited_user': ('django.db.models.fields.TextField', [], {'max_length': '30', 'blank': 'True'}),
            'enabled': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'sender': ('django.db.models.fields.EmailField', [], {'default': "''", 'max_length': '75', 'blank': 'True'}),
            'subject_template': ('django.db.models.fields.CharField', [], {'max_length': '2000'}),
            'type': ('django.db.models.fields.CharField', [], {'default': "'text/plain'", 'max_length': '20'})
        }
    }

    complete_apps = ['emailmessagetemplates']


from django.conf import settings
from django.db import models
from django.core.exceptions import ValidationError
from django.template import Context, Template, TemplateSyntaxError

if 'south' in settings.INSTALLED_APPS:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules([], ["^emailmessagetemplates\.fields\.SeparatedValuesField"])

class SeparatedValuesField(models.TextField):
    """
    Adapted from http://justcramer.com/2008/08/08/custom-fields-in-django/
    """
    __metaclass__ = models.SubfieldBase

    def __init__(self, *args, **kwargs):
        self.token = kwargs.pop('token', ',')
        super(SeparatedValuesField, self).__init__(*args, **kwargs)

    def to_python(self, value):
        if not value:
            return
        if isinstance(value, list):
            return value
        return value.split(self.token)

    def get_prep_value(self, value):
        if not value:
            return ''
        assert(isinstance(value, list) or isinstance(value, tuple))
        return self.token.join([unicode(s) for s in value])

    def value_to_string(self, obj):
        value = self._get_val_from_obj(obj)
        return ",".join(value)


def validate_template_syntax(value):
    """
    Ensure that there aren't any gross errors in a template string
    """
    try:
        Template(value).render(Context({}))
    except TemplateSyntaxError, e:
        raise ValidationError("Invalid Template Syntax: " + e.message)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import emailmessagetemplates.fields
import django.core.mail.message


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='EmailMessageTemplate',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=50)),
                ('object_id', models.PositiveIntegerField(null=True, verbose_name=b'Related Object ID', blank=True)),
                ('type', models.CharField(default=b'text/plain', help_text=b'Should email content generated by this template be sent as plain text or HTML?', max_length=20, choices=[(b'text/plain', b'Text'), (b'text/html', b'HTML')])),
                ('subject_template', models.CharField(max_length=2000, validators=[emailmessagetemplates.fields.validate_template_syntax])),
                ('body_template_html', models.TextField(blank=True, verbose_name=b'HTML body template', validators=[emailmessagetemplates.fields.validate_template_syntax])),
                ('autogenerate_text', models.BooleanField(default=True, help_text=b"If checked, this option will cause a plain text version of the HTML email to automatically be created when it is sent.  If you'd like to write the plain text template manually, uncheck this box.")),
                ('body_template', models.TextField(blank=True, validators=[emailmessagetemplates.fields.validate_template_syntax])),
                ('sender', models.EmailField(default=b'', help_text=b"The address this email should appear to be sent 'from'.  If blank, defaults to 'None'.", max_length=75, verbose_name=b"'From' Email", blank=True)),
                ('base_cc', emailmessagetemplates.fields.SeparatedValuesField(default=b'', help_text=b'An optional list of email addresses to be CCed when this template is sent (in addition to any addresses specified when the message is sent)', verbose_name=b'CC', blank=True)),
                ('base_bcc', emailmessagetemplates.fields.SeparatedValuesField(default=b'', help_text=b'An optional list of email addresses to be BCCed when this template is sent (in addition to any addresses specified when the message is sent)', verbose_name=b'BCC', blank=True)),
                ('description', models.TextField()),
                ('enabled', models.BooleanField(default=True, help_text=b'When unchecked, this email will not be sent.')),
                ('edited_date', models.DateTimeField(auto_now=True)),
                ('edited_user', models.TextField(max_length=30, editable=False, blank=True)),
                ('content_type', models.ForeignKey(verbose_name=b'Related Object Type', blank=True, to='contenttypes.ContentType', null=True)),
            ],
            options={
                'ordering': ('name',),
                'verbose_name': 'Email Template',
            },
            bases=(models.Model, django.core.mail.message.EmailMultiAlternatives),
        ),
        migrations.AlterUniqueTogether(
            name='emailmessagetemplate',
            unique_together=set([('name', 'content_type', 'object_id')]),
        ),
    ]



from datetime import datetime, timedelta

from django.core.management import call_command
from django.core import mail
from django.test import TestCase
from django.contrib.sites.models import Site
from django.contrib.contenttypes.models import ContentType
from django.template import Context
from django.core.exceptions import ValidationError
from django.conf import settings

from models import EmailMessageTemplate
from fields import validate_template_syntax
from utils import send_mail, send_mass_mail, mail_admins, mail_managers

class TemplateRetrievalTest(TestCase):
    """
    Ensure that template objects are retrieved correctly from the database based 
    on queries by name and object, respecting the enabled flag and the fallback 
    behavior for object-based queries.  Queries that do not match active 
    templates should raise exceptions.
    """
    fixtures = ['test_templates',]

    # Simple named template retrievals
    def test_retrieve_named_template(self):
        """Ensure the correct named template is returned"""
        template = EmailMessageTemplate.objects.get_template("Template 1")
        self.assertEqual(template.pk, 1)

    def test_retrieve_missing_named_template(self):
        """Ensure an exception is raised if the correct template is missing"""
        self.assertRaisesMessage(EmailMessageTemplate.DoesNotExist,
            "EmailMessageTemplate matching query does not exist.",
            lambda: EmailMessageTemplate.objects.get_template("Nonexistent Template"))

    def test_retrieve_disabled_named_template(self):
        """Ensure an exception is raised if the correct template is disabled"""
        self.assertRaisesMessage(EmailMessageTemplate.DoesNotExist,
            "EmailMessageTemplate matching query does not exist.",
            lambda: EmailMessageTemplate.objects.get_template("Template 3"))

    # Retrievals of templates with specified objects
    def test_retrieve_object_template(self):
        """"Ensure the correct template is returned when queried with name and object"""
        site = Site.objects.get(pk=1)
        template = EmailMessageTemplate.objects.get_template("Template 1", related_object=site)
        self.assertEqual(template.pk, 4)

    def test_retrieve_missing_object_template(self):
        """Ensure an exception is raised if a template specifed with bad name and object is missing"""
        site = Site.objects.get(pk=1)
        self.assertRaisesMessage(EmailMessageTemplate.DoesNotExist,
            "EmailMessageTemplate matching query does not exist.",
            lambda: EmailMessageTemplate.objects.get_template("Nonexistent Template", site))

    def test_retrieve_object_fallback_template(self):
        """
        Ensure the fallback (no object) template is returned if a template with
        the specifed object is missing
        """
        site = Site.objects.get(pk=2)
        template = EmailMessageTemplate.objects.get_template("Template 1", related_object=site)
        self.assertEqual(template.pk, 1)

    def test_retrieve_disabled_object_fallback_template(self):
        """
        Ensure the fallback (no object) template is returned if a template with
        the specifed object is disabled
        """
        site = Site.objects.get(pk=2)
        template = EmailMessageTemplate.objects.get_template("Template 2", related_object=site)
        self.assertEqual(template.pk, 2)


class TemplatePreparationTest(TestCase):
    """
    Ensure that template data is correctly produced when a template is 
    instantiated and prepared for sending.  This includes assembling the From, CC 
    and BCC address lists and rendering the subject and body templates. 
    """
    fixtures = ['test_templates',]
    
    def setUp(self):
        self.context = {'hello': '*HELLO*', 'world': '*WORLD*'}

    # Ensure the sender address is set correctly
    def test_default_from(self):
        """
        If not specified in the template or on the instance, the from address is 
        pulled from settings.  If specified, this will be the 
        EMAILMESSAGETEMPLATES_DEFAULT_FROM_EMAIL setting, else the DEFAULT_FROM_EMAIL 
        setting.
        """
        template = EmailMessageTemplate.objects.get_template("Template 1")
        with self.settings(DEFAULT_FROM_EMAIL='test1@example.com'):
            self.assertEqual(template.from_email, 'test1@example.com')
            with self.settings(EMAILMESSAGETEMPLATES_DEFAULT_FROM_EMAIL='test2@example.com'):
                self.assertEqual(template.from_email, 'test2@example.com')


    def test_template_specified_from(self):
        """
        If a from address is specified on the template, use that instead.
        """
        template = EmailMessageTemplate.objects.get_template("Template 2")
        with self.settings(DEFAULT_FROM_EMAIL='dontdoit@example.com'):
            self.assertEqual(template.from_email, 'example@example.com')

    def test_prepare_specified_from(self):
        """
        If a from address is specified on the template, use that instead.
        """
        template = EmailMessageTemplate.objects.get_template("Template 2")
        with self.settings(DEFAULT_FROM_EMAIL='dontdoit@example.com'):
            template.from_email='inprepare@example.com'
            self.assertEqual(template.from_email, 'inprepare@example.com')
    # Ensure the "to" address is set correctly

    def test_instance_to(self):
        """Ensure the "to" email set on the instance is used"""
        template = EmailMessageTemplate.objects.get_template("Template 2")
        template.to = ['inprepare@example.com']
        self.assertEqual(template.to, ['inprepare@example.com'])
    # Ensure the "CC" address list is set correctly

    def test_instance_cc(self):
        """Ensure the "cc" email list set o the instance is used"""
        template = EmailMessageTemplate.objects.get_template("Template 1")
        template.cc = ['inprepare@example.com', 'inprepare2@example.com']
        cc = template.cc
        cc.sort()
        self.assertEqual(cc, ['inprepare2@example.com', 'inprepare@example.com'])

    def test_template_cc(self):
        """
        Ensure the "cc" email list set in the template is used
        """
        template = EmailMessageTemplate.objects.get_template("Template 2")
        cc = template.cc
        cc.sort()
        self.assertEqual(cc, ['a@example.com', 'b@example.com'])

    def test_instance_template_cc(self):
        """
        Ensure the "cc" email list seton the instance is used in 
        conjunction with cc addresses in the template
        """
        template = EmailMessageTemplate.objects.get_template("Template 2")
        template.cc = ['inprepare@example.com', 'inprepare2@example.com']
        cc = template.cc
        cc.sort()
        self.assertEqual(cc, ['a@example.com', 'b@example.com',
                              'inprepare2@example.com', 'inprepare@example.com'])

    # Ensure the "BCC" address list is set correctly
    def test_instance_bcc(self):
        """Ensure the "bcc" email list set on the instance is used"""
        template = EmailMessageTemplate.objects.get_template("Template 1")
        template.bcc = ['inprepare@example.com', 'inprepare2@example.com']
        bcc = template.bcc
        bcc.sort()
        self.assertEqual(bcc, ['inprepare2@example.com', 'inprepare@example.com'])

    def test_template_bcc(self):
        """
        Ensure the "bcc" email list set in the template is used
        """
        template = EmailMessageTemplate.objects.get_template("Template 2")
        bcc = template.bcc
        bcc.sort()
        self.assertEqual(bcc, ['c@example.com', 'd@example.com'])

    def test_instance_template_bcc(self):
        """
        Ensure the "bcc" email list set on the instance is used in 
        conjunction with bcc addresses in the template
        """
        template = EmailMessageTemplate.objects.get_template("Template 2")
        template.bcc = ['inprepare@example.com', 'inprepare2@example.com']
        bcc = template.bcc
        bcc.sort()
        self.assertEqual(bcc, ['c@example.com', 'd@example.com',
                               'inprepare2@example.com', 'inprepare@example.com'])
    # Ensure the subject renders correctly

    def test_render_subject_template(self):
        """
        Ensure the subject renders as expected 
        """
        template = EmailMessageTemplate.objects.get_template("Template 1")
        template.context = self.context
        self.assertEqual(template.subject, "Test 1 Subject *HELLO*")

    def test_render_prefixed_subject_template(self):
        """
        Ensure the subject renders as expected with a specified prefix
        """
        template = EmailMessageTemplate.objects.get_template("Template 1")
        template.context = self.context
        template.subject_prefix = "[PREFIX] "
        self.assertEqual(template.subject, "[PREFIX] Test 1 Subject *HELLO*")
    # Ensure the body renders correctly

    def test_render_body_template(self):
        """
        Ensure the body renders as expected 
        """
        template = EmailMessageTemplate.objects.get_template("Template 1")
        template.context = self.context
        self.assertEqual(template.body, "Test 1 body *WORLD*")
    #Ensure the prepare method populates the message correctly

    def test_prepare_template(self):
        """
        Ensure the email parameters are set correctly by the prepare method 
        """
        template = EmailMessageTemplate.objects.get_template("Template 1")
        template.context=self.context
        template.from_email='from@example.com'
        template.to=['to@example.com']
        template.cc=['cc@example.com']
        template.bcc=['bcc@example.com']
        template.subject_prefix='[PREFIX] '

        self.assertEqual(template.subject, "[PREFIX] Test 1 Subject *HELLO*")
        self.assertEqual(template.body, "Test 1 body *WORLD*")
        self.assertEqual(template.from_email, 'from@example.com')
        self.assertEqual(template.to, ['to@example.com'])
        self.assertEqual(template.cc, ['cc@example.com'])
        self.assertEqual(template.bcc, ['bcc@example.com'])


class TemplateSendingTest(TestCase):
    """
    Ensure that the emails are actually composed and sent successfully
    """
    fixtures = ['test_templates',]
    
    def setUp(self):
        self.context = {'hello': '*HELLO*', 'world': '*WORLD*'}

    # Email sending
    def test_send_email_success(self):
        """Ensure that an email can be successfully sent to the outbox"""
        template = EmailMessageTemplate.objects.get_template("Template 1")
        template.context=self.context
        template.from_email='from@example.com'
        template.to=['to@example.com']
        template.cc=['cc@example.com']
        template.bcc=['bcc@example.com']
        template.subject_prefix='[PREFIX] '
        template.send()

        # Test that one message has been sent.
        self.assertEqual(len(mail.outbox), 1)

        # Verify that the subject of the first message is correct.
        self.assertEqual(mail.outbox[0].subject, '[PREFIX] Test 1 Subject *HELLO*')
        self.assertEqual(mail.outbox[0].body, "Test 1 body *WORLD*")
        self.assertEqual(mail.outbox[0].from_email, 'from@example.com')
        self.assertEqual(mail.outbox[0].to, ['to@example.com'])
        self.assertEqual(mail.outbox[0].cc, ['cc@example.com'])
        self.assertEqual(mail.outbox[0].bcc, ['bcc@example.com'])     

class HTMLTemplateTest(TestCase):
    """
    Ensure that HTML template data is correctly produced when a template is 
    instantiated and sent.  This includes rendering alternatives and 
    autogenerating content a required. 
    """
    fixtures = ['test_templates',]
    
    def setUp(self):
        self.context = {'hello': '*HELLO*', 'world': '*WORLD*'}
    
    def test_html_attachment(self):
        """Ensure that html content is included in HTML messages"""
        with self.settings(EMAILMESSAGETEMPLATES_ALLOW_HTML_MESSAGES=True):
            template = EmailMessageTemplate.objects.get_template("Template 5")
            template.context=self.context
            template.from_email='from@example.com'
            template.to=['to@example.com']
            template.send()
            
            # Test that one message has been sent.
            self.assertEqual(len(mail.outbox), 1)
            
            message = mail.outbox[0].message()
            
            #Test that the basic message setup is correct
            self.assertTrue(message.is_multipart())
            self.assertEqual(message.get_content_type(), 'multipart/alternative')
            self.assertEqual(message.get_default_type(), 'text/plain')
            
            #Test that there are two parts, with content types for text and html
            self.assertEqual(message.get_payload(0).get_content_type(), 'text/plain')
            self.assertEqual(message.get_payload(1).get_content_type(), 'text/html')
        
        
    def test_text_autogenreate(self):
        """
        Ensure that plain text content is correctly autogenerated from HTML 
        content
        """
        with self.settings(EMAILMESSAGETEMPLATES_ALLOW_HTML_MESSAGES=True): 
            template = EmailMessageTemplate.objects.get_template("Template 5")
            template.context=self.context
            template.from_email='from@example.com'
            template.to=['to@example.com']
            template.send()
            
            self.assertEqual(len(mail.outbox), 1)
            
            message = mail.outbox[0].message()
            
            self.assertTrue("""# *HELLO* *WORLD* in HTML!""" in message.get_payload(0).as_string())
            self.assertTrue("""<h1>*HELLO* *WORLD* in HTML!</h1>""" in message.get_payload(1).as_string())
    
    def test_html_disallowed(self):
        """
        Ensure that plain text content is is used when HTML emails are not 
        permitted by settings
        """
        with self.settings(EMAILMESSAGETEMPLATES_ALLOW_HTML_MESSAGES=False): 
            template = EmailMessageTemplate.objects.get_template("Template 6")
            template.context=self.context
            template.from_email='from@example.com'
            template.to=['to@example.com']
            template.send()
            
            self.assertEqual(len(mail.outbox), 1)
            
            message = mail.outbox[0].message()
            
            self.assertFalse(message.is_multipart())
            self.assertTrue("""*HELLO* *WORLD* in text!""" in message.as_string())
        
    def test_html_not_present(self):
        """
        Ensure that plain text messages don't automatically get HTML content
        """
        with self.settings(EMAILMESSAGETEMPLATES_ALLOW_HTML_MESSAGES=True):
            template = EmailMessageTemplate.objects.get_template("Template 1")
            template.context=self.context
            template.from_email='from@example.com'
            template.to=['to@example.com']
            template.send()
    
            # Test that one message has been sent.
            self.assertEqual(len(mail.outbox), 1)
            
            self.assertFalse(mail.outbox[0].message().is_multipart())


class TemplateValidatorTest(TestCase):
    """
    Ensure the template syntax validator correctly identifies invalid template 
    strings
    """

    def test_valid_template(self):
        """Ensure template validator passes valid templates"""
        validate_template_syntax("hello {% if world %} world {% else %} you {% "
                                 "endif %}")

    def test_invalid_template(self):
        """Ensure template validator rejects invalid templates"""
        self.assertRaises(ValidationError, 
            validate_template_syntax, 
            "hello {% if world %} world")


class UtilityFunctionTest(TestCase):
    """
    Test the behavior of the 4 utility functions: send_mail, send_mass_mail, 
    mail_admins, mail_managers
    """
    
    fixtures = ['test_templates',]
    
    def setUp(self):
        self.context = {'hello': '*HELLO*', 'world': '*WORLD*'}
        self.context2 = {'hello': '-GOODBYE-', 'world': '-EARTH-'}
 
    def test_send_mail(self):
        """Ensure the send_mail function works"""
        send_mail("Template 1", context=self.context, 
                  recipient_list=['to@example.com'])

        # Test that one message has been sent.
        self.assertEqual(len(mail.outbox), 1)

        # Verify that the subject of the first message is correct.
        self.assertEqual(mail.outbox[0].subject, 'Test 1 Subject *HELLO*')
        self.assertEqual(mail.outbox[0].body, "Test 1 body *WORLD*")
        self.assertEqual(mail.outbox[0].to, ['to@example.com'])

    def test_send_mass_mail(self):
        """Ensure the send_mass_mail function works"""

        datatuple = [(self.context,'from1@example.com',['to1@example.com']),
                     (self.context2,'from2@example.com',['to2@example.com']),]
        send_mass_mail("Template 1", datatuple=datatuple)

        # Test that one message has been sent.
        self.assertEqual(len(mail.outbox), 2)

        # Verify that the subject of the first message is correct.
        self.assertEqual(mail.outbox[0].subject, 'Test 1 Subject *HELLO*')
        self.assertEqual(mail.outbox[0].body, "Test 1 body *WORLD*")
        self.assertEqual(mail.outbox[0].to, ['to1@example.com'])

        self.assertEqual(mail.outbox[1].subject, 'Test 1 Subject -GOODBYE-')
        self.assertEqual(mail.outbox[1].body, "Test 1 body -EARTH-")
        self.assertEqual(mail.outbox[1].to, ['to2@example.com'])

    def test_mail_admins(self):
        """Ensure the mail_admins function works"""
        with self.settings(ADMINS=(('a','admin1@example.com'),
                                   ('b','admin2@example.com'))):
            mail_admins("Template 1", context=self.context)

        # Test that one message has been sent.
        self.assertEqual(len(mail.outbox), 1)

        # Verify that the subject of the first message is correct.
        self.assertEqual(mail.outbox[0].subject, 'Test 1 Subject *HELLO*')
        self.assertEqual(mail.outbox[0].body, "Test 1 body *WORLD*")
        self.assertEqual(mail.outbox[0].to, ['admin1@example.com', 
                                             'admin2@example.com'])

    def test_mail_managers(self):
        """Ensure the mail_managers function works"""
        with self.settings(MANAGERS=(('a','admin1@example.com'),
                                     ('b','admin2@example.com'))):
            mail_managers("Template 1", context=self.context)

        # Test that one message has been sent.
        self.assertEqual(len(mail.outbox), 1)

        # Verify that the subject of the first message is correct.
        self.assertEqual(mail.outbox[0].subject, 'Test 1 Subject *HELLO*')
        self.assertEqual(mail.outbox[0].body, "Test 1 body *WORLD*")
        self.assertEqual(mail.outbox[0].to, ['admin1@example.com', 
                                             'admin2@example.com'])



from django.db import models
from django.core.mail import EmailMultiAlternatives
from django.template import Context, Template
from django.contrib.contenttypes.models import ContentType

try:
    # Django>=1.7
    from django.contrib.contenttypes.fields import GenericForeignKey
except ImportError:
    from django.contrib.contenttypes.generic import GenericForeignKey
    
    

from conf import settings
from fields import SeparatedValuesField, validate_template_syntax

class EmailMessageTemplateManager(models.Manager):

    def get_template(self, name, related_object=None):
        """
        If a related object is specified, we'll first try to retrieve a template 
        that matches both the name and the object, and if none exists, we'll try 
        to retrieve a template with the specified name without a related object 
        instead (to support situations where some objects have a specialized 
        template, but, when none exists, we want to fall back to a default). 
        """
        if related_object:
            object_id = related_object.pk
            content_type = ContentType.objects.get_for_model(related_object)
        else:
            object_id = None
            content_type = None
        
        try:
            return self.get(name=name, object_id=object_id,
                            content_type=content_type, enabled=True)
        except EmailMessageTemplate.DoesNotExist:
            if not related_object:
                raise
            return self.get(name=name, object_id=None, content_type=None,
                            enabled=True)


class EmailMessageTemplate(models.Model, EmailMultiAlternatives):
    """
    A template for an email to be sent by the system.  Also a subclass of 
    Django's EmailMessage, so its methods can be used for sending.
    """
    
    CONTENT_TYPE_CHOICES = (('text/plain', 'Text',),
                            ('text/html', 'HTML',),)

    #Fields to identify a template
    name = models.CharField(max_length=50)
    content_type = models.ForeignKey(ContentType, null=True, blank=True, verbose_name="Related Object Type")
    object_id = models.PositiveIntegerField(null=True, blank=True, verbose_name="Related Object ID")
    related_object = GenericForeignKey('content_type', 'object_id')

    #Fields to override from EmailMessage
    type = models.CharField(max_length=20, choices=CONTENT_TYPE_CHOICES, default='text/plain', help_text="Should email content generated by this template be sent as plain text or HTML?")
    subject_template = models.CharField(max_length=2000, validators=[validate_template_syntax])
    body_template_html = models.TextField(validators=[validate_template_syntax], blank=True, verbose_name="HTML body template")
    autogenerate_text = models.BooleanField(default=True, help_text="If checked, this option will cause a plain text version of the HTML email to automatically be created when it is sent.  If you'd like to write the plain text template manually, uncheck this box.")
    body_template = models.TextField(validators=[validate_template_syntax], blank=True)
    sender = models.EmailField(max_length=75, blank=True, default='', verbose_name="'From' Email", help_text="The address this email should appear to be sent 'from'.  If blank, defaults to '{0}'.".format(settings.EMAILMESSAGETEMPLATES_DEFAULT_FROM_EMAIL))
    base_cc = SeparatedValuesField(blank=True, default='', verbose_name="CC", help_text="An optional list of email addresses to be CCed when this template is sent (in addition to any addresses specified when the message is sent)")
    base_bcc = SeparatedValuesField(blank=True, default='', verbose_name="BCC", help_text="An optional list of email addresses to be BCCed when this template is sent (in addition to any addresses specified when the message is sent)")

    #Other information
    description = models.TextField()
    enabled = models.BooleanField(default=True, help_text="When unchecked, this email will not be sent.")
    edited_date = models.DateTimeField(auto_now=True, editable=False, blank=True)
    edited_user = models.TextField(max_length=30, editable=False, blank=True)

    objects = EmailMessageTemplateManager()

    #Ensure compatibility with EmailMessage CC, BCC, and from_email data
    _instance_to = []
    _instance_cc = []
    _instance_bcc = []
    _instance_from = None

    @property
    def to(self):
        """
        The recipient addresses specified on the instance.
        """
        return self._instance_to

    @to.setter
    def to(self, value):
        """
        Set the recipient addresses on the instance.
        """
        self._instance_to = value

    @property
    def cc(self):
        """
        The unique set of CC addresses specified either in the template or on 
        the instance.
        """
        return list(set(self._instance_cc) | set(self.base_cc if self.base_cc is not None else []))

    @cc.setter
    def cc(self, value):
        """
        Add any addresses not in the template's CC list to the instance list.
        """
        self._instance_cc = list(set(value) - set(self.base_cc if self.base_cc is not None else []))

    @property
    def bcc(self):
        """
        The unique set of BCC addresses specified either in the template or on 
        the instance.
        """
        return list(set(self._instance_bcc) | set(self.base_bcc if self.base_bcc is not None else []))

    @bcc.setter
    def bcc(self, value):
        """
        Add any addresses not in the template's BCC list to the instance list.
        """
        self._instance_bcc = list(set(value) - set(self.base_bcc if self.base_bcc is not None else []))

    @property
    def from_email(self):
        """
        Use the specified sender if set, and then fall back to the template 
        sender, and finally the setting value.
        """
        return self._instance_from or self.sender or\
            settings.EMAILMESSAGETEMPLATES_DEFAULT_FROM_EMAIL or\
            settings.DEFAULT_FROM_EMAIL

    @from_email.setter
    def from_email(self, value):
        """
        Set the sender addresses on the instance.
        """
        self._instance_from = value

    def __unicode__(self):
        status = " (Disabled)" if not self.enabled else ""
        if self.related_object:
            return "{0} for {1}{2}".format(self.name, self.related_object, status)
        return self.name + status

    # Preparing and sending messages
    _context = Context({})
    subject_prefix = ""

    @property
    def context(self):
        return self._context

    @context.setter
    def context(self, value):
        if issubclass(type(value), Context):
            self._context = value
        else:
            self._context = Context(value)

    @property
    def subject(self):
        return self.subject_prefix + Template(self.subject_template).render(self.context)

    @subject.setter
    def subject(self, value):
        """
        No-op to prevent EmailMessage from stomping on the template 
        """
        pass

    @property
    def body(self):
        if self.is_html_message() and self.autogenerate_text:
            try:
                import html2text
                return html2text.html2text(self.html_content())
            except ImportError:
                pass
        return Template(self.body_template).render(self.context)

    @body.setter
    def body(self, value):
        """
        No-op to prevent EmailMessage from stomping on the template 
        """
        pass
    
    def html_content(self):
        """
        Render the HTML message content, if any
        """
        if self.is_html_message():
            return Template(self.body_template_html).render(self.context)
        return None
    
    def is_html_message(self):
        return settings.EMAILMESSAGETEMPLATES_ALLOW_HTML_MESSAGES \
            and self.type == 'text/html'
        
                        
    def send(self, fail_silently=False):
        """
        Sends the email message.
        """
        result = None
        send_error = None
        try:
            html_content = self.html_content()
            if html_content is not None:
                self.attach_alternative(html_content, "text/html")
            result = super(EmailMessageTemplate, self).send(fail_silently=False)
        except Exception as e:
            raise
            send_error = e

        #Raise an exception if the user has requested it
        if not fail_silently and send_error:
            raise send_error

        return result
    
    def related_item_display(self):
        return unicode(self.related_object) if self.related_object else 'None'
    related_item_display.short_description = "Related Item"

    def __init__(self,*args,**kwargs):
        """
        Initialize the template, ensuring that the from_email is not set 
        unnecessarily
        """
        super(EmailMessageTemplate, self).__init__(*args,**kwargs)
        #resetting sender to default so EmailMessage won't stomp on it
        self.from_email = None
        

    class Meta:
        ordering = ('name',)
        unique_together = (("name", "content_type", "object_id"),)
        verbose_name = "Email Template"
        app_label = "emailmessagetemplates"
